#!/usr/local/bin/perl 
#
#  lbadmin.cgi (c) Marc Majcher 2000
#

use strict;
use CGI ":standard";
use Lifebox;
use CGI::Carp qw(fatalsToBrowser);

#  Define lookup table for action parameter processing

my %admin_actions = (
		     entry         => \&entry,
		     prefs         => \&prefs,
		     addentry      => \&addentry,
		     changeprefs   => \&changeprefs,
		     xslt          => \&xslt,
		     writexslt     => \&writexslt,
		     template      => \&template,
		     writetemplate => \&writetemplate,
		     index         => \&index,
		     writeindex    => \&writeindex,
		     );


my $bodytext = <<CONTENT;
<tr>
    <td colspan="2" width="640">
    <center>
    <h1><a href="lbadmin.cgi">Life Box Admin</a></h1>
    </center>
    </td>
</tr>
CONTENT

#  Process the actions, and check for a password

if (param()) {
    if (my $action_result = &{$admin_actions{param('action')}}) {
	$bodytext .= $action_result;
    }
    else {

	$bodytext .= <<MESS;
<tr>
    <td colspan="2">
	<p><b>Bad password.</b></p>
    </td>
</tr>

MESS
    }
}
else {
    $bodytext .= <<INTRO;
    <tr><td colspan="2">

    <p><a href="lbadmin.cgi?action=entry"><b>Add New Entry</b></a></p>

    <p><a href="lbadmin.cgi?action=prefs">Edit Preferences</a></p>

    <p><a href="lbadmin.cgi?action=index">Edit Index Page</a></p>

    <p><a href="lbadmin.cgi?action=xslt">Edit XSLT Stylesheet</a></p>
    <p><a href="lbadmin.cgi?action=template">Edit HTML Template</a></p>

    <p><form action="lbadmin.cgi" method="post">
	Edit Entry:<br />
<select name="entry">
INTRO

    for my $page (sort &Lifebox::Entry::pages) {
	my $entry = &Lifebox::Entry::read($page);
	my $title = $entry->title;
	$bodytext .= "<option value=\"$page\">$page - $title";
    }

    $bodytext .= <<INTRO;
</select>
	<input type="hidden" name="action" value="entry">
	<input type="submit" value="Edit">
    </form></p>
	    
    </td></tr>
INTRO

}


&Lifebox::print_template($bodytext);



#
#  Begin subroutines
#

sub entry {

    #  Edit an entry - detect if we're trying to edit an existing
    #  entry, and if so, throw down the goods for that.

    my ($entry, $time, $title, $content);
    my $bodytext;

    if ($entry = param('entry')) {
	my $o_entry = &Lifebox::Entry::read($entry);
	$time = $o_entry->time;
	$title = $o_entry->title;
	$content = $o_entry->xml;

	$bodytext = "<tr><td colspan=\"2\">Editing entry \#$entry\:</td></tr>";
    }
    else {
	$time = time();
	$content = "<entry>\nfoobar\n</entry>";
    }


    $bodytext = <<FORM;

<tr>
    <form action="lbadmin.cgi" method="post">
    <td align="right">Password:</td>
    <td align="left"> <input type="password" name="password" /></td>
</tr>
<tr>
    <td align="right">Time:</td>
    <td align="left"> <input type="text" name="time" value="$time" /></td>
</tr>
<tr>
    <td align="right">Title:</td>
    <td align="left"> <input type="text" name="title" size="40" value="$title" /></td>
</tr>
<tr>
    <td align="left" colspan="2">Entry:<br />
    <textarea cols="70" rows="50" name="entry">
$content
</textarea>
    </td>
</tr>
<tr>
    <td colspan="2"><input type="submit" value="Create Page" /></td>
    <input type="hidden" name="action" value="addentry" />
    </form>
</tr>
FORM

    return $bodytext;
}

sub addentry {

    #  Add an entry to the database.  This is the same whether
    #  we're creating a new entry, or updating an old one.

    return unless &Lifebox::goodpass(param('password'));

    my $time = param('time');
    my $title = param('title');
    my $entry = param('entry');

    my $o_entry = Lifebox::Entry->new;
    $o_entry->time($time);
    $o_entry->title($title);
    $o_entry->xml($entry);

    $o_entry->write;
    
    my $bodytext = <<CONF;
    <tr>
    <td colspan="2">Entry \#$time (<b>$title</b>) submitted successfully.  
    <a href="lifebox.cgi?action=display&page=$time">View</a></td>
    </tr>
CONF

    return $bodytext;
}

sub prefs {

    #  Print out the preference editing form

    my %prefs = &Lifebox::get_prefs;

    my $bodytext = <<PREFS;
    <form action="lbadmin.cgi" method="post">
    <input type="hidden" name="action" value="changeprefs">

    <tr>
    <td align="right"><b>Enter Password:</b></td>
    <td align="left"><input type="password" size="20" name="password"></td>
    </tr>

    <tr>
    <td align="right">New Password:</td>
    <td align="left"><input type="password" size="20" name="newpassword"></td>
    </tr>
    
    <tr>
    <td align="right">Verify Password:</td>
    <td align="left"><input type="password" size="20" name="verpassword"></td>
    </tr>
    
    <tr>
    <td align="right">Your Email Address:</td>
    <td align="left"><input type="text" size="20" name="email" value="$prefs{email}"></td>
    </tr>

    <tr>
    <td align="right">Page Background Color:</td>
    <td align="left"><input type="text" size="20" name="bgcolor" value="$prefs{bgcolor}"></td>
    </tr>

    <tr>
    <td align="right">Table Background Color:</td>
    <td align="left"><input type="text" size="20" name="tablecolor" value="$prefs{tablecolor}"></td>
    </tr>

    <tr>
    <td align="right">Text Color:</td>
    <td align="left"><input type="text" size="20" name="text" value="$prefs{text}"></td>
    </tr>

    <tr>
    <td align="right">Link Color:</td>
    <td align="left"><input type="text" size="20" name="link" value="$prefs{link}"></td>
    </tr>

    <tr>
    <td align="right">Followed Link Color:</td>
    <td align="left"><input type="text" size="20" name="vlink" value="$prefs{vlink}"></td>
    </tr>

    <tr>
    <td align="right">Active Link Color:</td>
    <td align="left"><input type="text" size="20" name="alink" value="$prefs{alink}"></td>
    </tr>

    <tr>
    <td align="right">Default Page Title:</td>
    <td align="left"><input type="text" size="20" name="title" value="$prefs{title}"></td>
    </tr>

    <tr>
    <td align="right">Default Text Font:</td>
    <td align="left"><input type="text" size="20" name="font" value="$prefs{font}"></td>
    </tr>

    <tr>
	<td>&nbsp;</td>
	<td align="left"><input type="submit" value="Set Prefs"></td>
    </tr>
    </form>
PREFS

    return $bodytext;
}

sub changeprefs {

    #  Update the preference database and print a confirmation

    return unless &Lifebox::goodpass(param('password'));

    my $bodytext;
    my %prefs = (
		 link       => param('link'),
		 text       => param('text'),
		 alink      => param('alink'),
		 tablecolor => param('tablecolor'),
		 bgcolor    => param('bgcolor'),
		 vlink      => param('vlink'),
		 title      => param('title'),
		 font       => param('font'),
		 email      => param('email'),
		 );

    if (param('newpassword')) {
	if (param('newpassword') eq param('verpassword')) {
	    $prefs{password} = param('newpassword');
	}
	else {
	    $bodytext .= <<ERROR;
   <tr>
      <td colspan="2"><b>New passwords don't match.  Password not set.</b></td>
    </tr>
ERROR
	}
    }

    &Lifebox::set_prefs(%prefs);

    $bodytext .= <<DONE;
    <tr>
	<td colspan="2">Preferences set.  <a href="lbadmin.cgi">Back</a></td>
    </tr>
DONE
}

sub xslt {

    #  Edit the XSLT stylesheet - print the form to edit with the 
    #  current stylesheet in it.

    my $stylesheet = &Lifebox::get_xslt;

    my $bodytext = <<FORM;
<tr><td colspan="2"><h3>Edit XSLT Stylesheet</h3>
(See <a href="http://metalab.unc.edu/xml/books/bible/updates/14.html" target="_new">the XML Bible</a> for help.)</td></tr>
<tr>
    <form action="lbadmin.cgi" method="post">
    <input type="hidden" name="action" value="writexslt">
    <td align="right">Password:</td>
    <td align="left"> <input type="password" name="password" /></td>
</tr>
<tr>
    <td align="left" colspan="2">Stylesheet:<br />
    <textarea cols="70" rows="50" name="stylesheet" wrap="wrap">
$stylesheet
</textarea>
    </td>
</tr>
<tr>
    <td colspan="2"><input type="submit" value="Write Stylesheet" /></td>
    </form>
</tr>
FORM

    return $bodytext;
}

sub writexslt {

    #  Writes the XSLT stylesheet to the file and prints a confirmation.

    return unless &Lifebox::goodpass(param('password'));

    my $stylesheet = param('stylesheet');

    #  handle some textarea and xml weirdness

    $stylesheet =~ s/\cM\cJ/\n/g;
    $stylesheet =~ s/\&/\&amp\;/g;
    $stylesheet .= "\n" unless ($stylesheet =~ /\n$/);

    &Lifebox::write_xslt($stylesheet);

    my $bodytext = <<CONF;
    <tr>
    <td colspan="2">XSLT stylesheet written successfully.  
    <a href="lifebox.cgi?action=latest">Test</a></td>
    </tr>
CONF

    return $bodytext;
}

sub template {

    #  Edit the HTML template - print the form to edit with the 
    #  current template in it.

    my $template = &Lifebox::get_template;

    my $bodytext = <<FORM;
<tr><td colspan="2"><h3>Edit HTML Template</h3>
(See <a href="http://search.cpan.org/doc/MJD/Text-Template-1.23/Template.pm" target="_new">the Text::Template perldoc</a> for help.)</td></tr>
<tr>
    <form action="lbadmin.cgi" method="post">
    <input type="hidden" name="action" value="writetemplate">
    <td align="right">Password:</td>
    <td align="left"> <input type="password" name="password" /></td>
</tr>
<tr>
    <td align="left" colspan="2">Template:<br />
    <textarea cols="70" rows="50" name="template" wrap="wrap">
$template
</textarea>
    </td>
</tr>
<tr>
    <td colspan="2"><input type="submit" value="Write Template" /></td>
    </form>
</tr>
FORM

    return $bodytext;

}

sub writetemplate {

    #  Writes the HTML template to the file and prints a confirmation.

    return unless &Lifebox::goodpass(param('password'));

    my $template = param('template');

    #  handle some textarea and xml weirdness

    $template =~ s/\cM\cJ/\n/g;
    $template =~ s/\&/\&amp\;/g;
    $template .= "\n" unless ($template =~ /\n$/);

    &Lifebox::write_template($template);

    my $bodytext = <<CONF;
    <tr>
    <td colspan="2">HTML template written successfully.  
    <a href="lifebox.cgi?action=latest">Test</a></td>
    </tr>
CONF

    return $bodytext;
}


sub index {

    #  Edit the index page - print the form to edit with the 
    #  current index in it.

    my $index = &Lifebox::get_index;

    my $bodytext = <<FORM;
<tr><td colspan="2"><h3>Edit Index Page</h3></td></tr>
<tr>
    <form action="lbadmin.cgi" method="post">
    <input type="hidden" name="action" value="writeindex">
    <td align="right">Password:</td>
    <td align="left"> <input type="password" name="password" /></td>
</tr>
<tr>
    <td align="left" colspan="2">Index:<br />
    <textarea cols="70" rows="50" name="index" wrap="wrap">
$index
</textarea>
    </td>
</tr>
<tr>
    <td colspan="2"><input type="submit" value="Write Index" /></td>
    </form>
</tr>
FORM

    return $bodytext;

}

sub writeindex {

    #  Writes the index page to the file and prints a confirmation.

    return unless &Lifebox::goodpass(param('password'));

    my $index = param('index');

    #  handle some textarea and xml weirdness

    $index =~ s/\cM\cJ/\n/g;
    $index =~ s/\&/\&amp\;/g;
    $index .= "\n" unless ($index =~ /\n$/);

    &Lifebox::write_index($index);

    my $bodytext = <<CONF;
    <tr>
    <td colspan="2">Index page written successfully.  
    <a href="lifebox.cgi">Test</a></td>
    </tr>
CONF

    return $bodytext;
}

